import { Injectable } from '@nestjs/common';
import { BaseDataService } from 'src/core/modules/data/service/base-data.service';
import { InjectRepository } from '@nestjs/typeorm';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { Repository } from 'typeorm';
import { QueueTicket } from '../../domain/entities/ticket.entity';
import { QueueOrderModel, QueueTicketModel } from '../models/queue.model';
import { QueueOrder } from '../../domain/entities/order.entity';

@Injectable()
export class TicketDataService extends BaseDataService<QueueTicket> {
  constructor(
    @InjectRepository(QueueTicketModel, CONNECTION_NAME.DEFAULT)
    private repo: Repository<QueueTicketModel>,

    @InjectRepository(QueueOrderModel, CONNECTION_NAME.DEFAULT)
    private order: Repository<QueueOrderModel>,
  ) {
    super(repo);
  }

  async createQueueOrder(order: QueueOrder): Promise<QueueOrderModel> {
    return await this.order.save(order);
  }

  async loginQueue(id: string): Promise<QueueOrder> {
    return this.order.findOneOrFail({
      relations: ['tickets'],
      where: {
        transaction_id: id,
      },
    });
  }

  async queueTickets(order_id: string): Promise<QueueTicketModel[]> {
    return this.repo.find({
      relations: ['items', 'items.item', 'items.item.item_queue'],
      where: {
        order_id,
      },
    });
  }

  async queueTicketItems(
    order_id: string,
    ticket_id: string,
  ): Promise<QueueTicketModel[]> {
    return this.repo.find({
      relations: ['items', 'items.item', 'items.item.item_queue'],
      where: {
        order_id,
        id: ticket_id,
      },
    });
  }

  async queueItemTickets(
    order_id: string,
    item_id: string,
  ): Promise<QueueTicketModel[]> {
    return this.repo.find({
      relations: ['items', 'items.item', 'items.item.item_queue'],
      where: {
        order_id,
        items: [{ item_id }, { item: { item_queue: { id: item_id } } }],
      },
    });
  }
}
